/*******************************************************************************
  LCDemoNoiascaSerialEeprom

  Receive text to be displayed over Serial and store text to EEPROM
  Retrive text from EEPROM on startup

  credits to MajicDesigns for the idea of his MD_MAX72xx_Message_Serial.ino
  
  name=LCDemoNoiascaSerialEeprom
  version=1.0.1
  author=Werner Rothschopf
  date=2020-03-08
 *******************************************************************************/

//We always have to include the library
#include <NoiascaLedControl.h>        // get the library here http://werner.rothschopf.net/201904_arduino_ledcontrol_max7219.htm
#include <EEPROM.h>

/*  ***** These pin numbers will probably need to be adopted to your hardware ***** */
const uint8_t LEDCS_PIN = 8;           // 8 LED CS or LOAD -  8 CS
const uint8_t LEDCLK_PIN = 13;         // 7 LED Clock      - 13 CLK on UNO/NANO
const uint8_t LEDDATA_PIN = 11;        // 9 LED DATA IN    - 11 MOSI on UNO/NANO
const uint8_t LED_MODULES = 1;     

LedControl lc = LedControl  (LEDDATA_PIN, LEDCLK_PIN, LEDCS_PIN, LED_MODULES);  // Software Bitbang - use this if you can't use Hardware SPI

const unsigned long delaylong = 4000;       // waiting time between displayed text - milliseconds

// Global message buffers shared by Serial and Scrolling functions
#define BUF_SIZE  64                  // could be reduced if less then 64 characters are needed or LED_MODULES * 8 
char curMessage[BUF_SIZE];            // current Message to be displayed
char newMessage[BUF_SIZE];            // Buffer for new message comming in
bool newMessageAvailable = false;     // flag if new Message is available
const uint16_t addrMessage = 10;      // EEPROM start address where to store the message


/*
  Check Serial for new data
*/
void readSerial()
{
  static uint8_t  putIndex = 0;
  while (Serial.available())
  {
    newMessage[putIndex] = (char)Serial.read();
    if ((newMessage[putIndex] == '\n') || (putIndex >= BUF_SIZE - 3)) // end of message character or full buffer
    {
      // put in a message separator and end the string
      newMessage[putIndex++] = ' ';
      newMessage[putIndex] = '\0';
      // restart the index for next filling spree and flag we have a message waiting
      putIndex = 0;
      newMessageAvailable = true;
      EEPROM.put(addrMessage, newMessage);
    }
    else if (newMessage[putIndex] != '\r')
      // Just save the next char in next location
      putIndex++;
  }
}


/*
  Print new text to display 
  simple start without any fancy scrolling so far.
*/
void updateText()
{
  if (newMessageAvailable)
  {
    Serial.print(F("Print new message to display:")); Serial.println(newMessage);
    strcpy(curMessage, newMessage);
    lc.clearDisplay();
    lc.print(curMessage);
    newMessageAvailable=false;
  }
}


/*
  Print a welcome message to Display
*/
void printWelcomeOn7Segment() {
  Serial.println(F("Print a welcome message to Display"));
  lc.clearDisplay();
  lc.print("HELLO U2");
  delay(delaylong);
}


void setup() {
  Serial.begin(115200);
  Serial.println(F("Noiasca Led Control Serial EEPROM"));
  /* The MAX72XX needs to initialize hardware */
  lc.begin();
  //Setup routines for MAX72xx
  for (byte i = 0; i < LED_MODULES; i++)
  {
    lc.shutdown(i, false);                // The MAX72xx is in power-saving mode on startup, we have to do a wakeup call
    lc.setIntensity(i, 8);                // Set the brightness to a medium value, possible is 0-15 
  }
  lc.clearDisplay();                      // and clear all displays

  // print welcome
  printWelcomeOn7Segment();

  // Read stored text from EEPROM and print to display
  EEPROM.get(addrMessage, newMessage);   
  newMessageAvailable = true;            
  updateText();                          // hint: if you use the sketch the first time and your EEPROM is not set, the display might get dark after the welcome message, if no printable characters are found
  Serial.println(F("\nnow, enter new text in Serial Monitor \n- to be displayed on LED \n- and to be stored to EEPROM for next startup "));
}


void loop() 
{
  readSerial();
  updateText();
}
