/*
 *    NoiascaLedControlBase.cpp - A library for controling Leds with a MAX7219/MAX7221
 *    
 *    noiasca
 *    https://werner.rothschopf.net/201904_arduino_ledcontrol_max7219.htm
 *
 *    Permission is hereby granted, free of charge, to any person
 *    obtaining a copy of this software and associated documentation
 *    files (the "Software"), to deal in the Software without
 *    restriction, including without limitation the rights to use,
 *    copy, modify, merge, publish, distribute, sublicense, and/or sell
 *    copies of the Software, and to permit persons to whom the
 *    Software is furnished to do so, subject to the following
 *    conditions:
 * 
 *    This permission notice shall be included in all copies or 
 *    substantial portions of the Software.
 * 
 *    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 *    EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 *    OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 *    NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 *    HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 *    WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 *    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 *    OTHER DEALINGS IN THE SOFTWARE.
 
      2021-11-14
 
 */

#include <NoiascaLedControlCommon.h>

/*
//the opcodes for the MAX7221 and MAX7219 - MISSING final delete in 1.2.5 or higher
#define OP_NOOP   0
#define OP_DIGIT0 1
#define OP_DIGIT1 2
#define OP_DIGIT2 3
#define OP_DIGIT3 4
#define OP_DIGIT4 5
#define OP_DIGIT5 6
#define OP_DIGIT6 7
#define OP_DIGIT7 8
#define OP_DECODEMODE  9
#define OP_INTENSITY   10
#define OP_SCANLIMIT   11
#define OP_SHUTDOWN    12
#define OP_DISPLAYTEST 15
*/

LedControl_base::LedControl_base(uint8_t numDevices) :
  maxDevices(numDevices)
  {}

uint8_t LedControl_base::getDeviceCount() {
  return maxDevices;
}

void LedControl_base::shutdown(uint8_t addr, bool b) {
  if (!isInitialized) begin();  // backward compatibility: if user has forgotten begin method, call it
  if(addr >= maxDevices)
    return;
  if(b)
    spiTransfer(addr, OP_SHUTDOWN, 0);
  else
    spiTransfer(addr, OP_SHUTDOWN, 1);
}

void LedControl_base::setScanLimit(uint8_t addr, uint8_t limit) {
  if(addr >= maxDevices)
      return;
  if(limit < 8)
    spiTransfer(addr, OP_SCANLIMIT, limit);
}

void LedControl_base::setIntensity(uint8_t addr, uint8_t intensity) {
  if(addr >= maxDevices)
    return;
  if(intensity < 16)	
    spiTransfer(addr, OP_INTENSITY, intensity);
}

void LedControl_base::clearDisplay(uint8_t addr) {
  uint8_t offset;
  if(addr >= maxDevices)
    return;
  offset = addr * 8;
  for(uint8_t i = 0; i < 8; i++) {
    status[offset + i]=0;
    spiTransfer(addr, i + 1, status[offset + i]);
  }
  setCursor(addr, 0);      // clearDisplay sets cursor to begin of device
}
void LedControl_base::clearDisplay() {
  for(uint8_t i = 0; i< maxDevices; i++) {
      clearDisplay(i);
  }
  setCursor(0, 0);         // clearDisplay sets cursor to begin of first device
}

void LedControl_base::setLed(uint8_t addr, uint8_t row, uint8_t column, boolean state) {
  uint8_t offset;
  uint8_t val = 0x00;
  if(addr >= maxDevices)
    return;
  if(row > 7 || column > 7)
    return;
  offset = addr * 8;
  val = 0b10000000 >> column;  // was B10000000
  if(state)
    status[offset + row] = status[offset + row] | val;
  else {
    val = ~val;
    status[offset + row] = status[offset + row] & val;
  }
  spiTransfer(addr, row + 1, status[offset + row]);
}

void LedControl_base::setRow(uint8_t addr, uint8_t row, uint8_t value) {
  uint8_t offset;
  if(addr >= maxDevices)
    return;
  if(row > 7)
    return;
  offset = addr * 8;
  status[offset + row] = value;
  spiTransfer(addr, row + 1, status[offset + row]);
}

void LedControl_base::setColumn(uint8_t addr, uint8_t col, uint8_t value) {
  uint8_t val;
  if(addr >= maxDevices)
    return;
  if(col > 7) 
    return;
  for(uint8_t row=0; row < 8;row++) {
    val=value >> (7 - row);
    val=val & 0x01;
    setLed(addr, row, col, val);
  }
}

void LedControl_base::setDigit(uint8_t addr, uint8_t digit, uint8_t value, boolean dp) {
  uint8_t offset;
  uint8_t v;
  if(addr >= maxDevices)
    return;
  if(digit > 7 || value > 15)
    return;
  offset = addr * 8;
  v=pgm_read_byte_near(charTable + value); 
  if(dp)
    v |= 0b10000000;             // was   B10000000
  status[offset + digit] = v;
  spiTransfer(addr, digit+1, v);
}

void LedControl_base::setChar(uint8_t addr, uint8_t digit, char value, boolean dp) {
  uint8_t offset;
  uint8_t index, v;
  if(addr >= maxDevices)
    return;
  if(digit > 7)
    return;
  offset = addr * 8;
  index = (uint8_t)value;
  if(index > 127 ) {
    //no defined beyond index 127, so we use the space char
    index = 32;
  }  
  v = pgm_read_byte_near(charTable + index); 
  if(dp)
    v |= 0b10000000;              // was B10000000
  status[offset + digit] =v;
  spiTransfer(addr, digit + 1, v);
}

//*** noiasca's new methods ***
void LedControl_base::setCursor(uint8_t newPosition){
  currentPosition = newPosition;
}

void LedControl_base::setCursor(uint8_t addr, uint8_t devPosition){
  currentPosition = addr * 8 + devPosition;   // Position in device (order 01234567!) (not the old style digit!)
}

void LedControl_base::setEndOfDevice(optionEndOfDevice newOption)
{
  endOfDevice = newOption;
}

void LedControl_base::handleEndOfDevice(uint8_t addr){            
  switch(endOfDevice)
  {
    case THIS_DEVICE:                    // jump to digit7 on that Adress
      currentPosition = addr * 8; 
      break;
    case FIRST_DEVICE:                   // jump to adress 0 digit 7 (=position 0) - MISSING: eigentlich sinnarm, kann fast immer durch NEXT_DEVICE ersetzt werden
      currentPosition = 0; 
      break;
    case NEXT_DEVICE:                    // jump to next adress digit 7 or back to position 0 if we are currently the last device
      //Serial.print(F("\nD243 addr:")); Serial.print(addr);
      if (addr == maxDevices - 1)
        {currentPosition = 0;}
      else
        {currentPosition = addr + 1 * 8;}
      //Serial.print(F(" currentPosition:"));Serial.println(currentPosition);
      break;
    case SHIFT_LEFT: 
       //implementation missing for shift to left
    break;
  }  
}

//noiasca's virtual method to write (print, println) characters
size_t LedControl_base::write(uint8_t value) {
  uint8_t addr = currentPosition / 8;
  uint8_t digit = 7 - (currentPosition % 8); // invers because digits are in this order: 7 6 5 4 3 2 1 0

  //Serial.print("\ncurrentPosition:"); Serial.print(currentPosition);
  //Serial.print(" addr:"); Serial.print(addr);
  //Serial.print(" digit:"); Serial.print(digit);
  //Serial.print(" value:"); Serial.print(value);

  if (value == 10)                                            // \n linefeed
  {
    handleEndOfDevice(addr); 
  }
  else if (value == 13)                                       // \r carriage return MISSING: eventuell alles unter 32 nicht ausgeben
  {
    // don't print carriage return
  }  
  else if (value == '.' || value == ',')                    // MISSING: Prüfung auf digit < 7 ausgebaut
  {
    if (currentPosition == 0)    // if on position 0, print just the dot 
    {
      setRow(0, 7, 0b10000000);  //add decimal point to the last printed digit
      lastPosition = currentPosition;
      currentPosition++;
    }
    else
    {
      uint8_t lastAddr = lastPosition / 8;
      uint8_t lastDigit = 7 - (lastPosition % 8);
      setRow(lastAddr, lastDigit, status[lastAddr * 8 + lastDigit] | 0b10000000);  //add decimal point to the last printed digit
      //setRow(addr, digit + 1, status[addr * 8 + digit + 1] | 0b10000000);        //add decimal point to the heading digit
    }
  }
  else 
  {
    setChar(addr, digit, value, 0);
    lastPosition = currentPosition;
    currentPosition++;
    if (digit == 0) handleEndOfDevice(addr);
  }
  return 1; // assume success
}

// fix for vt table for MKRZERO
void LedControl_base::spiTransfer(uint8_t addr, volatile uint8_t opcode, volatile uint8_t data){
  (void) addr;
  (void) opcode;
  (void) data;
}

void LedControl_base::begin(){
}
