/*
 *    NoiascaLedControlSpi.h - A library for controling LEDss with a MAX7219/MAX7221
 *    
 *    noiasca
 *    https://werner.rothschopf.net/201904_arduino_ledcontrol_max7219.htm
 * 
 *    Permission is hereby granted, free of charge, to any person
 *    obtaining a copy of this software and associated documentation
 *    files (the "Software"), to deal in the Software without
 *    restriction, including without limitation the rights to use,
 *    copy, modify, merge, publish, distribute, sublicense, and/or sell
 *    copies of the Software, and to permit persons to whom the
 *    Software is furnished to do so, subject to the following
 *    conditions:
 * 
 *    This permission notice shall be included in all copies or 
 *    substantial portions of the Software.
 * 
 *    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 *    EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 *    OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 *    NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 *    HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 *    WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 *    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 *    OTHER DEALINGS IN THE SOFTWARE.
 
 *    Version
 *    2021-02-28 noiasca 1.2.4  cleanup name conflict with NanoBLE33
 *    2022-01-09 noiasca        removed include SPI.h from .cpp
 *    2020-12-08 noiasca        split into abstract base class and hw implementation
 *    2020-08-08 noiasca        no hardware calls in constructor, begin() method has to be called separatly, soft and hardware SPI splitted
 *    2020-03-07 noiasca        add hardware SPI
 *    2020-03-06 noiasca        rebuild int to uint8_t
 *    2019-04-25 noiasca        inherit print class
 *    2007       Eberhard Fahle 
 
 */

 /*  Extensions by noiasca:
 
 The modules/devices digits are addressed as following:
 
 device   0                 1                 2                       7
 addr     0 0 0 0 0 0 0 0   1 1 1 1 1 1 1 1   2 2 2 2 2 2 2 2         7 7 7 7 7 7 7 7
 digit    7 6 5 4 3 2 1 0   7 6 5 4 3 2 1 0   7 6 5 4 3 2 1 0   ...   7 6 5 4 3 2 1 0
 
 position 0 1 2 3 4 5 6 7   8 9 10            16                      56            63

 offene Punkte 
 - EndOfDevice                          0                                  1                   2             3
 - Bei "Adress-Ende":                   Beginne Am Anfang dieser Adresse | Beginne bei 0 / 7 | Shift Links | Weiter in nächste Adresse |  
 - Bei Ende letzte Adrese (Zeilenende): Beginne Am Anfang dieser Adresse | Beginne bei 0 / 7 | Shift Links |
 - Schreiben über das Ende eines Devices (über das Ende der Device-Kette)ermöglichen (Position 254??)

 */
 
#pragma once
#define NoiascaLedControlSpi_h

#if defined(__AVR__)
#include <avr/pgmspace.h>
#elif defined(ESP8266)
#include <pgmspace.h>
#endif

#if (ARDUINO >= 100)
#include <Arduino.h>
//#include <WProgram.h>
#else
#endif

#include <Print.h>
#include <NoiascaLedControlCommon.h>
#include <SPI.h>             // MISSING: todo: remove in Version higher than 1.2.3, replace with externals

/** \brief class for MAX7219/MAX7221 (Hardware SPI)

    this class should be used whenever you can use Hardware SPI.
*/
class LedControlSpi : public LedControl_base {
  protected :
    const uint8_t csPin;          /* This one is driven LOW for chip selectzion. former SPI_CS */
    void spiTransfer(uint8_t addr, uint8_t opcode, uint8_t data);     /* Send out a single command to the device */

  public:
    /** \brief Create a new MAX7219/MAX7221 object for Hardware SPI
      
      \param csPin pin for selecting the device 
      \param numDevices	maximum number of devices that can be controlled
     */
    LedControlSpi(uint8_t csPin, uint8_t numDevices = 1);

     /** \brief initialise hardware
     
       call this method in your setup().
       initialise hardware - sets pinMode for csPin        
     */       
    void begin();                  
};